(ns com.oly.static-sites.do-blog.pages.chess.movement)

(defn line-horizontal [[x y] size]
  (loop [step 0
         cords [[0 1 y]]]
    (if (< (-> cords last second) size) 
      (recur (+ step 1) (conj cords [step (+ 1 (-> cords last second)) y]))
      cords)))

(defn line-vertical [[x y] size]
  (loop [step 0
         cords [[0 x 1]]]
    (if (< (-> cords last last) size) 
      (recur (+ step 1) (conj cords [step x (+ 1 (-> cords last last))]))
      cords)))

(defn line-projection [[x y] size]
  (concat (line-horizontal [x y] size)
          (line-vertical [x y] size)))

(defn edge?
  "Return true if x and are inside the grid"
  [[_ x y]]
   (and (and (< x 9) (< y 9))
             (and (> x 0) (> y 0))))

(defn diagonal-ascending
"down and to the right"
  [[x y] size]
  (loop [step 0
         cords [[0 x y]]]
    (if (or (edge? (or (-> cords pop peek) (-> cords last)) )  (edge? (-> cords last)))
      (recur (+ step 1)
             ;; generate 2 coordinates either side of actual position
             (conj cords
                   [step (+ x step)
                    (+ y step)]
                   [step (- x step)
                    (- y step)]))
      cords)))

(defn diagonal-descending [[x y] size]
  (loop [step 0
         cords [[0 x y]]]
    (if (or (edge? (or (-> cords pop peek) (-> cords last)) )
             (edge? (-> cords last)))
      (recur (+ step 1)
             ;; generate 2 coordinates either side of actual position
             (conj cords
                   [step (+ x step)
                    (- y step)]
                   [step (- x step)
                    (+ y step)]))
      cords)))

(defn diagonal-projection [[x y] size]
  (concat (diagonal-ascending [x y] size)
          (diagonal-descending [x y] size)))

(defn directional-projection [[x y] size]
  (concat (diagonal-projection [x y] size)
          (line-projection [x y] size)))

(defn star [[x y] size]
  (loop [step 1
         cords [[0 x y]]]
    (case step
      3
      (recur (+ step 1)
             (conj cords
                   ;[(- (-> cords first second) s) (-> cords first last)]
                   [step (- (-> cords first second) 2)  (+ (-> cords first last) 1)]
                   [step (- (-> cords first second) 2)  (- (-> cords first last) 1)]

                   ;[(+ (-> cords first second) s) (-> cords first last)]
                   [step (+ (-> cords first second) 2)  (- (-> cords first last) 1)]
                   [step (+ (-> cords first second) 2)  (+ (-> cords first last) 1)]

                   [step (+ (-> cords first second) 1) (- (-> cords first last) 2)]
                   [step (- (-> cords first second) 1) (- (-> cords first last) 2)]

                   [step (- (-> cords first second) 1) (+ (-> cords first last) 2)]
                   [step (+ (-> cords first second) 1) (+ (-> cords first last) 2)]))
      4 cords
      (recur (+ step 1)
             cords
             #_(conj cords
                   [step (- (-> cords first second) s) (-> cords first last)]
                   [(+ (-> cords first second) s) (-> cords first last)]
                   [(-> cords first second)  (- (-> cords first last) s)]
                   [(-> cords first second)  (+ (-> cords first last) s)])))))

(defn all-around-projection [[x y] size]
  [[0 (- x 1) (+ 1 y)] [0 x (+ 1 y)] [0 (+ 1 x) (+ 1 y)]
   [0 (- x 1) y] [0 (+ 1 x) y]
   [0 (- x 1) (- y 1)] [0 x (- y 1)] [0 (+ 1 x) (- y 1)]])

(defn pawn-projection [[x y] direction size]
  (if (or (= y 2) (= y 7))
    [[0 x (+ y direction)] [1 x (+ y (* 2 direction))]]
    [[0 x (+ y direction)]]))

(defn fetch-selected-piece-moves [[pos piece]]
  (case (:type piece)
    :king (all-around-projection (:pos piece) 8)
    :queen (directional-projection (:pos piece) 8)
    :rook (line-projection (:pos piece) 8)
    :knight (star (:pos piece) 8)
    :pawn (pawn-projection (:pos piece) (if (= :white (:colour piece)) 1 -1) 8)
    :bishop (diagonal-projection (:pos piece) 8)
    (prn (str "type = " (:type piece)))))

(defn valid-move? [dest-pos game-state]
  (let [piece (first (filter (fn [[k v]] (:selected v)) game-state))
        moves (fetch-selected-piece-moves piece)]
    ;; if selected and destintion pos are the same no need to move
    (when (not= dest-pos (:pos piece))
      (some (fn [p] (= p dest-pos)) moves))))

;; (line-horizontal [8 1] 8)
;; (line-vertical [8 1] 8)

;; ;(line-projection [8 1] 8)
;; (directional-projection [1 1] 8)
;; (directional-projection [8 1] 8)
;; (star [8 1] 8)
;; (directional-projection [5 1] 8)

;; (deftest line-projection-test
;;   (testing "Horizontal direction"
;;     (is (= (line-horizontal [2 3] 8)
;;            [[1 3] [2 3] [3 3] [4 3] [5 3] [6 3] [7 3] [8 3]])))
;;   (testing "Vertical direction"
;;     (is (= (line-vertical [2 3] 8)
;;            [[2 1] [2 2] [2 3] [2 4] [2 5] [2 6] [2 7] [2 8]]))))

;; (deftest diagonal-projection-test
;;   (testing "Diagonal ascending direction"
;;     (is (= (diagonal-ascending [2 3] 8)
;;            [[1 2] [2 3] [3 4] [4 5] [5 6] [6 7] [7 8]])))
;;   (testing "Diagonal descending direction"
;;     (is (= (diagonal-descending [2 3] 8)
;;            [[8 7] [7 6] [6 5] [5 4] [4 3] [3 2] [2 1]])))

;;   (testing "Diagonals "
;;     (is (= (diagonal-projection [2 3] 8)
;;            [[1 2] [2 3] [3 4] [4 5] [5 6] [6 7] [7 8]]))))
